/// <reference types="node" />
/** @ignore */
export declare const decodeUtf8: (input?: ArrayBuffer | NodeJS.ArrayBufferView | null | undefined, options?: {
    stream?: boolean | undefined;
} | undefined) => string;
/** @ignore */
export declare const encodeUtf8: (value?: string) => Uint8Array;
