/// <reference types="node" />
import { Readable, ReadableOptions as ReadableOptions_ } from 'node:stream';
/** @ignore */
type ReadableOptions = ReadableOptions_;
/** @ignore */
export declare function toNodeStream<T>(source: Iterable<T> | AsyncIterable<T>, options?: ReadableOptions): Readable;
export {};
